package strsets;

import static myutil.BinSearch.binarySearch;

/**
 * Insieme di stringhe ordinarto e modificabile implementato con un array
 * Estende ArrayStrSet e implementa ModifiablesSorteStrSet
 */
public class ArraySortedStrSet extends ArrayStrSet
                            implements ModifiableSortedStrSet {
                                
    // Non definisce costruttori
    // Eredita il costruttore di default da ArrayStrSet

    /**
     * Aggiunge una stringa all'insieme (se non gia' presente)
     * Se la stringa e' gia' presente, lascia l'insieme immutato.
     * (sovrascrive il metodo della superclasse per tener conto dell'ordinamento)
     * 
     * @param s     la stringa
     * @return      true se la stringa non e' contenuta ed e' stata aggiunta
     *              correttamente, false altrimenti
     */ 
    public boolean add(String s) {
        int j = binarySearch(strArray, 0, strCount, s);
        if (j < 0) {        // non presente
            j = -(j + 1);   // punto d'inserimento
            if (strArray.length == strCount) {  // se l'array e' pieno...
                int newSize = (int) Math.ceil(EXPFACTOR*(strCount + 1));
                                                // nuova capacita'
                String[] newA = new String[newSize];                // crea copia
                System.arraycopy(strArray, 0, newA, 0, strCount);   // con nuova
                strArray = newA;                                    // capacita'
            }
            for (int i = strCount - 1 ; i >= j ; i--)   // sposta gli elementi oltre
                strArray[i + 1] = strArray[i];          // il punto d'inserimento
            strArray[j] = s;                            // inserisce
            strCount++;                                 // nuovo num elementi
            return true;
        } else
            return false;
    }
    
    /**
     * Rimunove una stringa dall'insieme (se presente)
     * (sovrascrive il metodo della superclasse per tener conto dell'ordianmento)
     * 
     * @param s     la stringa
     * @return      true se la stringa e' contenuta ed e' stata rimossa
     *              correttamente, false altrimenti
     */
    public boolean remove(String s) {
        int j = binarySearch(strArray, 0, strCount, s);
        if (j >= 0) {   // presente
            for (int i = j + 1 ; i < strCount ; i++)    // sposta gli elementi dopo
                strArray[i - 1] = strArray[i];          // la stringa da rimuovere
            strCount--;                                 // nuovo num elementi
            return true;
        } else
            return false;
    }
    
    /**
     * Verifica se una stringa e' contenuta nell'insieme
     * (sovrascrive il metodo della superclasse per tener conto dell'ordianmento)
     * 
     * @param s     la stringa
     * @return      true se la stringa e' contenuta, false altrimenti
     */
    public boolean contains(String s) {
        return (binarySearch(strArray, 0, strCount, s) >= 0);
    }

    /**
     * Ritorna la k-esima stringa dell'insieme ordinato.
     * (implementazione di un metodo dell'interfaccia)
     * 
     * @param k     la posizione della stringa
     * @return      La stringa nella k-esima posizione.
     *              Se k < 0 oppure k > size - 1 ritorna null.
     */
    public String getKth(int k) {
        if (k < 0 || k >= strCount)
            return null;
        return strArray[k];
    }
}