package strsets;

/**
 * Insieme di stringhe modificabile implementato con una lista
 * Estende StrSet
 */
public class LinkedStrSet implements ModifiableStrSet {
    
    /**
     * Classse nidificata statica per rappresentare gli elementi della lista
     */
    private static class Elem {
        private String str;     // stringa dell'elemento
        private Elem next;      // riferimento al prossimo elemento della lista
        
        /**
         * Costruisce un'istanza dell'elemento
         * 
         * @param s     stringa da inserire nell'elemento
         * @param n     coda della lista
         * @return      nuova istanza
         */
        private Elem(String s, Elem n) {
            str = s;
            next = n;
        }
    }
    
    private Elem first;    // primo elemento (testa) della lista
    
    /**
     * Metodo ausiliario che aggiunge un nuovo elemento in testa alla lista
     * 
     * @param s     stringa da inserire
     */
    private void addElem(String s) {
        Elem newN = new Elem(s, first);
        first = newN;
    }
    
    /**
     * Costruisce un insieme con le stringhe in un array dato
     * 
     * @param sA    array di stringhe
     * @return      il nuovo insieme
     */
    public LinkedStrSet(String[] sA) {
        first = null;
        for (int i = 0 ; i < sA.length ; i++)
            addElem(sA[i]);
    }
        
    /**
     * Aggiunge una stringa all'insieme (se non gia' presente)
     * Se la stringa e' gia' presente, lascia l'insieme immutato.
     * (implementazione di un metodo dell'interfaccia)
     * 
     * @param s     la stringa
     * @return      true se la stringa non e' contenuta ed e' stata aggiunta
     *              correttamente, false altrimenti
     */ 
    public boolean add(String s) {
        if (!contains(s)) {
            addElem(s);
            return true;
        } else
            return false;
    }

    /**
     * Rimunove una stringa dall'insieme (se presente)
     * (implementazione di un metodo dell'interfaccia)
     * 
     * @param s     la stringa
     * @return      true se la stringa e' contenuta ed e' stata rimossa
     *              correttamente, false altrimenti
     */
    public boolean remove(String s) {
        Elem prev = null, p = first;                // scorre la lista mantenendo
        while (p != null && !p.str.equals(s)) {     // in prev il riferimento
            prev = p;                               //  all'elemento precedente
            p = p.next;
        }
        if (p != null) {                            // se la stringa e' presente...
            if (prev == null)                       // sgancia l'elemento dalla lista
                first = first.next;
            else
                prev.next = p.next;
            return true;
        } else
            return false;
    }

    /**
     * Verifica se una stringa e' contenuta nell'insieme
     * (implementazione di un metodo dell'interfaccia)
     * 
     * @param s     la stringa
     * @return      true se la stringa e' contenuta, false altrimenti
     */
    public boolean contains(String s) {
        Elem p = first;
        while (p != null && !p.str.equals(s))
            p = p.next;
        return (p != null);
    }
    
    /**
     * Dimensione dell'insieme
     * (implementazione di un metodo dell'interfaccia)
     * 
     * @return      il numero di stringhe nell'insieme
     */
    public int size() {
        int count = 0;
        Elem p = first;
        while (p != null) {
            count++;
            p = p.next;
        }
        return count;
    }
}